subroutine evolve(g11,g12,g13,g22,g23,g33,                         &
            g11m1,g12m1,g13m1,g22m1,g23m1,g33m1,                   &
            k11,k12,k13,k21,k22,k23,k31,k32,k33,                   &
            k11m1,k12m1,k13m1,k21m1,k22m1,k23m1,k31m1,k32m1,k33m1, &
            alpha,beta1,beta2,beta3,                               &
            nx,ny,nz,h,time,dt,nt)

use types
use hpfprocessors
use evol_interface 

real(SP), dimension(:,:,:) :: g11,g12,g13,g22,g23,g33
real(SP), dimension(:,:,:) :: g11m1,g12m1,g13m1,g22m1,g23m1,g33m1
real(SP), dimension(:,:,:) :: k11,k12,k13,k21,k22,k23,k31,k32,k33
real(SP), dimension(:,:,:) :: k11m1,k12m1,k13m1,k21m1,k22m1,k23m1, &
                                 k31m1,k32m1,k33m1
real(SP), dimension(:,:,:) :: alpha,beta1,beta2,beta3

!HPF$ DISTRIBUTE g11 *(*,BLOCK,BLOCK) ONTO *PROC
!HPF$ ALIGN WITH *g11 :: g12,g13,g22,g23,g33
!HPF$ ALIGN WITH *g11 :: g11m1,g12m1,g13m1,g22m1,g23m1,g33m1
!HPF$ ALIGN WITH *g11 :: k11,k12,k13,k21,k22,k23,k31,k32,k33
!HPF$ ALIGN WITH *g11 :: k11m1,k12m1,k13m1,k21m1,k22m1,k23m1,k31m1,k32m1,k33m1
!HPF$ ALIGN WITH *g11 :: alpha,beta1,beta2,beta3

integer(I4B) nx,ny,nz,nt
real(SP) h,time,dt
real(SP) :: tol=100.0

real(SP), dimension(size(g11,1),size(g11,2),size(g11,3)) ::  &
                        gup11,gup12,gup13,gup22,gup23,gup33, &
                        c111,c112,c113,c122,c123,c133,       & 
                        c211,c212,c213,c222,c223,c233,       & 
                        c311,c312,c313,c322,c323,c333,       &
                        r11,r12,r13,r22,r23,r33,             &
                        t7,t8,t9,temp1

!HPF$ ALIGN WITH g11 :: gup11,gup12,gup13,gup22,gup23,gup33  
!HPF$ ALIGN WITH g11 :: c111,c112,c113,c122,c123,c133 
!HPF$ ALIGN WITH g11 :: c211,c212,c213,c222,c223,c233
!HPF$ ALIGN WITH g11 :: c311,c312,c313,c322,c323,c333 
!HPF$ ALIGN WITH g11 :: r11,r12,r13,r22,r23,r33
!HPF$ ALIGN WITH g11 :: t7,t8,t9,temp1

real(SP) ::  hamnorm,mom1norm,mom2norm,mom3norm 

! the evolution sequence is:
! evolve * swap * tensors
! while tensors = inverse * connect * ricci is input for evolve
! therefore tensors must be called once before the evolution loop

write(6,*) 'start evolve'

call tensors(g11,g12,g13,g22,g23,g33,          &
             gup11,gup12,gup13,gup22,gup23,gup33,                   &
             c111,c112,c113,c122,c123,c133,                         & 
             c211,c212,c213,c222,c223,c233,                         & 
             c311,c312,c313,c322,c323,c333,                         &
             r11,r12,r13,r22,r23,r33,                               &
             nx,ny,nz,h)

! constraints monitors
write(6,*) 'call constraints'

call  constraint(nx,ny,nz,                             &
                k11,k12,k13,k21,k22,k23,k31,k32,k33,   &
                c111,c112,c113,c122,c123,c133,         &
                c211,c212,c213,c222,c223,c233,         &
                c311,c312,c313,c322,c323,c333,         &
                r11,r12,r13,r22,r23,r33,               &
                gup11,gup12,gup13,gup22,gup23,gup33,   &
                hamnorm,mom1norm,mom2norm,mom3norm,h) 

it=0
write(6,*) 'Iter   Time    HamErr    Mom1Err    Mom2Err    Mom3Err'
write(6,'(i5,f9.5,1P,4e12.3)') it,time, hamnorm,mom1norm,mom2norm,mom3norm
 
!---------------------- EVOLUTION LOOP ------------------------------------

do it=1,nt
 time=time+dt

 call evolve_g(g11,g12,g13,g22,g23,g33,                             &
             g11m1,g12m1,g13m1,g22m1,g23m1,g33m1,                   &
             c111,c112,c113,c122,c123,c133,                         & 
             c211,c212,c213,c222,c223,c233,                         & 
             c311,c312,c313,c322,c323,c333,                         &
             k11,k12,k13,k21,k22,k23,k31,k32,k33,                   &
             alpha, beta1,beta2,beta3, h, dt, nx, ny, nz)

 call evolve_k(gup11,gup12,gup13,gup22,gup23,gup33,                 &
             k11,k12,k13,k21,k22,k23,k31,k32,k33,                   &
             k11m1,k12m1,k13m1,k21m1,k22m1,k23m1,k31m1,k32m1,k33m1, &
             c111,c112,c113,c122,c123,c133,                         &
             c211,c212,c213,c222,c223,c233,                         &
             c311,c312,c313,c322,c323,c333,                         &
             r11,r12,r13,r22,r23,r33,                               & 
             alpha, beta1,beta2,beta3,                              &
             nx,ny,nz,h, dt)


!swap time levels

	call swap(g11,g11m1)
	call swap(g12,g12m1)
	call swap(g13,g13m1)
	call swap(g22,g22m1)
	call swap(g23,g23m1)
	call swap(g33,g33m1)

	call swap(k11,k11m1)
	call swap(k12,k12m1)
	call swap(k13,k13m1)
	call swap(k21,k21m1)
	call swap(k22,k22m1)
	call swap(k23,k23m1)
	call swap(k31,k31m1)
	call swap(k32,k32m1)
	call swap(k33,k33m1)

 call tensors(g11,g12,g13,g22,g23,g33,                              &
             gup11,gup12,gup13,gup22,gup23,gup33,                   &
             c111,c112,c113,c122,c123,c133,                         & 
             c211,c212,c213,c222,c223,c233,                         & 
             c311,c312,c313,c322,c323,c333,                         &
             r11,r12,r13,r22,r23,r33,                               &
             nx,ny,nz,h)

 call  constraint(nx,ny,nz,                            &
                k11,k12,k13,k21,k22,k23,k31,k32,k33,   &
                c111,c112,c113,c122,c123,c133,         &
                c211,c212,c213,c222,c223,c233,         &
                c311,c312,c313,c322,c323,c333,         &
                r11,r12,r13,r22,r23,r33,               &
                gup11,gup12,gup13,gup22,gup23,gup33,   &
                hamnorm,mom1norm,mom2norm,mom3norm,h) 


 write(6,'(i5,f9.5,1P,4e12.3)') it,time, hamnorm,mom1norm,mom2norm,mom3norm

 if(hamnorm.gt.tol) exit
enddo

return
end





