subroutine initialize(g11,g12,g13,g22,g23,g33,                     &
            g11m1,g12m1,g13m1,g22m1,g23m1,g33m1,                   &
            k11,k12,k13,k21,k22,k23,k31,k32,k33,                   &
            k11m1,k12m1,k13m1,k21m1,k22m1,k23m1,k31m1,k32m1,k33m1, &
            alpha,beta1,beta2,beta3,                               &
            nx,ny,nz,h,dt,time)

use types
use hpfprocessors
use evol_interface

real(SP), dimension(:,:,:),intent(INOUT):: g11,g12,g13,g22,g23,g33
real(SP), dimension(:,:,:),intent(INOUT):: g11m1,g12m1,g13m1,g22m1,g23m1,g33m1
real(SP), dimension(:,:,:),intent(INOUT):: k11,k12,k13,k21,k22,k23,k31,k32,k33
real(SP), dimension(:,:,:),intent(INOUT):: k11m1,k12m1,k13m1,k21m1,k22m1, &
                                           k23m1,k31m1,k32m1,k33m1
real(SP), dimension(:,:,:),intent(IN)   :: alpha,beta1,beta2,beta3

!HPF$ DISTRIBUTE g11 *(*,BLOCK,BLOCK) ONTO *PROC
!HPF$ ALIGN WITH *g11 :: g12,g13,g22,g23,g33
!HPF$ ALIGN WITH *g11 :: g11m1,g12m1,g13m1,g22m1,g23m1,g33m1
!HPF$ ALIGN WITH *g11 :: k11,k12,k13,k21,k22,k23,k31,k32,k33
!HPF$ ALIGN WITH *g11 :: k11m1,k12m1,k13m1,k21m1,k22m1,k23m1,k31m1,k32m1,k33m1 
!HPF$ ALIGN WITH *g11 :: alpha, beta1, beta2, beta3

integer(I4B)  nx, ny, nz
real(SP) h,time,dt

real(SP), dimension(size(g11,1),size(g11,2),size(g11,3)) ::  &
                        gup11,gup12,gup13,gup22,gup23,gup33, &
                        c111,c112,c113,c122,c123,c133,       & 
                        c211,c212,c213,c222,c223,c233,       & 
                        c311,c312,c313,c322,c323,c333,       &
                        r11,r12,r13,r22,r23,r33

!HPF$ ALIGN WITH g11 :: gup11,gup12,gup13,gup22,gup23,gup33  
!HPF$ ALIGN WITH g11 :: c111,c112,c113,c122,c123,c133 
!HPF$ ALIGN WITH g11 :: c211,c212,c213,c222,c223,c233
!HPF$ ALIGN WITH g11 :: c311,c312,c313,c322,c323,c333 
!HPF$ ALIGN WITH g11 :: r11,r12,r13,r22,r23,r33

real(SP) dtsave,timesave

call tensors(g11,g12,g13,g22,g23,g33,                               &
             gup11,gup12,gup13,gup22,gup23,gup33,                   &
             c111,c112,c113,c122,c123,c133,                         & 
             c211,c212,c213,c222,c223,c233,                         & 
             c311,c312,c313,c322,c323,c333,                         &
             r11,r12,r13,r22,r23,r33,                               &
             nx,ny,nz,h)

dtsave   = dt
timesave = time
dt       = -0.5d0 * dt

!take Euler step backwards to generate n+1 level

dt       = 0.5d0 * dt

g11m1=g11
g12m1=g12
g13m1=g13
g22m1=g22
g23m1=g23
g33m1=g33

k11m1=k11
k12m1=k12
k13m1=k13
k21m1=k21
k22m1=k22
k23m1=k23
k31m1=k31
k32m1=k32
k33m1=k33

call evolve_g(g11,g12,g13,g22,g23,g33,                              &
             g11m1,g12m1,g13m1,g22m1,g23m1,g33m1,                   &
             c111,c112,c113,c122,c123,c133,                         & 
             c211,c212,c213,c222,c223,c233,                         & 
             c311,c312,c313,c322,c323,c333,                         &
             k11,k12,k13,k21,k22,k23,k31,k32,k33,                   &
             alpha, beta1,beta2,beta3, h, dt, nx, ny, nz)

call evolve_k(gup11,gup12,gup13,gup22,gup23,gup33,                  &
             k11,k12,k13,k21,k22,k23,k31,k32,k33,                   &
             k11m1,k12m1,k13m1,k21m1,k22m1,k23m1,k31m1,k32m1,k33m1, &
             c111,c112,c113,c122,c123,c133,                         &
             c211,c212,c213,c222,c223,c233,                         &
             c311,c312,c313,c322,c323,c333,                         &
             r11,r12,r13,r22,r23,r33,                               & 
             alpha, beta1,beta2,beta3,                              &
             nx,ny,nz,h, dt)

!swap time levels

call swap(g11,g11m1)
call swap(g12,g12m1)
call swap(g13,g13m1)
call swap(g22,g22m1)
call swap(g23,g23m1)
call swap(g33,g33m1)

call swap(k11,k11m1)
call swap(k12,k12m1)
call swap(k13,k13m1)
call swap(k21,k21m1)
call swap(k22,k22m1)
call swap(k23,k23m1)
call swap(k31,k31m1)
call swap(k32,k32m1)
call swap(k33,k33m1)

dt = 2.0d0*dt

!then continue with standard three-level scheme

time = time + dt

call tensors(g11,g12,g13,g22,g23,g33,                               &
             gup11,gup12,gup13,gup22,gup23,gup33,                   &
             c111,c112,c113,c122,c123,c133,                         & 
             c211,c212,c213,c222,c223,c233,                         & 
             c311,c312,c313,c322,c323,c333,                         &
             r11,r12,r13,r22,r23,r33,                               &
             nx,ny,nz,h)

call evolve_g(g11,g12,g13,g22,g23,g33,                              &
             g11m1,g12m1,g13m1,g22m1,g23m1,g33m1,                   &
             c111,c112,c113,c122,c123,c133,                         & 
             c211,c212,c213,c222,c223,c233,                         & 
             c311,c312,c313,c322,c323,c333,                         &
             k11,k12,k13,k21,k22,k23,k31,k32,k33,                   &
             alpha, beta1,beta2,beta3, h, dt, nx, ny, nz)

call evolve_k(gup11,gup12,gup13,gup22,gup23,gup33,                  &
             k11,k12,k13,k21,k22,k23,k31,k32,k33,                   &
             k11m1,k12m1,k13m1,k21m1,k22m1,k23m1,k31m1,k32m1,k33m1, &
             c111,c112,c113,c122,c123,c133,                         &
             c211,c212,c213,c222,c223,c233,                         &
             c311,c312,c313,c322,c323,c333,                         &
             r11,r12,r13,r22,r23,r33,                               & 
             alpha, beta1,beta2,beta3,                              &
             nx,ny,nz,h, dt)

!restore times and time levels

dt   = dtsave
time = timesave

return
end

















